@@ -38,6 +38,7 @@
 #include "clipboard-utils.h"
 #include "file-operation-utils.h"
 #include "file-operation-manager.h" //FileOpInfo
+#include "audio-play-manager.h"
 
 #include "file-utils.h"
 #include "bookmark-manager.h"
@@ -51,6 +52,10 @@
 #include "file-lauch-dialog.h"
 
 #include "file-operation-error-dialog.h"
+#include "file-enumerator.h"
+#include "gerror-wrapper.h"
+
+#include "global-settings.h"
 
 #include <QDesktopServices>
 #include <QUrl>
@@ -62,6 +67,7 @@
 
 #include <QLocale>
 #include <QStandardPaths>
+#include <recent-vfs-manager.h>
 
 #include <QDebug>
 
@@ -92,60 +98,94 @@ DirectoryViewMenu::DirectoryViewMenu(FMWindowIface *window, QWidget *parent) : Q
 
 void DirectoryViewMenu::fillActions()
 {
-    if (m_directory == "computer:///")
+    if (m_directory == "computer:///") {
         m_is_computer = true;
+    }
 
-    if (m_directory == "trash:///")
+    if (m_directory == "trash:///") {
         m_is_trash = true;
+    }
 
-    if (m_directory.startsWith("search://"))
+    if (m_directory.startsWith("search://")) {
         m_is_search = true;
+    }
 
-    if (m_directory.startsWith("burn://"))
+    if (m_directory.startsWith("burn://")) {
         m_is_cd = true;
+    }
+
+    if (m_directory.startsWith("recent://")) {
+        m_is_recent = true;
+    }
+
+    if (m_directory.startsWith("favorite://")) {
+        m_is_favorite = true;
+    }
+
+    if (m_directory.startsWith("kydroid:///")) {
+        m_is_kydroid = true;
+    }
+
+    QString desktop = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
+
+    for (auto uriIndex = 0; uriIndex < m_selections.count(); ++uriIndex) {
+        qDebug() << desktop;
+        if (m_selections.at(uriIndex) == "favorite:///?schema=trash"
+                || m_selections.at(uriIndex) == "favorite:///?schema=recent"
+                || m_selections.at(uriIndex) == "favorite://" + desktop + "?schema=file") {
+            m_can_delete = false;
+            break;
+        }
+    }
 
     //add open actions
     auto openActions = constructOpenOpActions();
     if (!openActions.isEmpty())
         addSeparator();
 
-    //create template actions
-    auto templateActions = constructCreateTemplateActions();
-    if (!templateActions.isEmpty())
-        addSeparator();
+    if (! m_is_kydroid){
+        //create template actions
+        auto templateActions = constructCreateTemplateActions();
+        if (!templateActions.isEmpty())
+            addSeparator();
 
-    //add view actions
-    auto viewActions = constructViewOpActions();
-    if (!viewActions.isEmpty())
-        addSeparator();
+        //add view actions
+        auto viewActions = constructViewOpActions();
+        if (!viewActions.isEmpty())
+            addSeparator();
+    }
 
     //add operation actions
     auto fileOpActions = constructFileOpActions();
     if (!fileOpActions.isEmpty())
         addSeparator();
 
-    //add plugin actions
-    auto pluginActions = constructMenuPluginActions();
-    if (!pluginActions.isEmpty())
-        addSeparator();
+    if (! m_is_kydroid){
+        //add plugin actions
+        auto pluginActions = constructMenuPluginActions();
+        if (!pluginActions.isEmpty())
+            addSeparator();
+    }
 
     //add propertries actions
     auto propertiesAction = constructFilePropertiesActions();
     if (!propertiesAction.isEmpty())
         addSeparator();
 
-    //add actions in computer:///
-    auto computerActions = constructComputerActions();
-    if (!computerActions.isEmpty())
-        addSeparator();
+    if (! m_is_kydroid){
+        //add actions in computer:///
+        auto computerActions = constructComputerActions();
+        if (!computerActions.isEmpty())
+            addSeparator();
 
-    //add actions in trash:///
-    auto trashActions = constructTrashActions();
-    if (!trashActions.isEmpty())
-        addSeparator();
+        //add actions in trash:///
+        auto trashActions = constructTrashActions();
+        if (!trashActions.isEmpty())
+            addSeparator();
 
-    //add actions in search:///
-    auto searchActions = constructSearchActions();
+        //add actions in search:///
+        auto searchActions = constructSearchActions();
+    }
 }
 
 const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
@@ -156,7 +196,7 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
 
     bool isBackgroundMenu = m_selections.isEmpty();
     if (isBackgroundMenu) {
-        l<<addAction(QIcon::fromTheme("window-new-symbolic"), tr("Open in &New Window"));
+        l<<addAction(QIcon::fromTheme("window-new-symbolic"), tr("Open in New Window"));
         connect(l.last(), &QAction::triggered, [=]() {
             auto windowIface = m_top_window->create(m_directory);
             auto newWindow = dynamic_cast<QWidget *>(windowIface);
@@ -164,7 +204,7 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
             //FIXME: show when prepared?
             newWindow->show();
         });
-        l<<addAction(QIcon::fromTheme("tab-new-symbolic"), tr("Open in New &Tab"));
+        l<<addAction(QIcon::fromTheme("tab-new-symbolic"), tr("Open in New Tab"));
         connect(l.last(), &QAction::triggered, [=]() {
             if (!m_top_window)
                 return;
@@ -176,6 +216,7 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
         if (m_selections.count() == 1) {
             auto info = FileInfo::fromUri(m_selections.first());
             auto displayName = info->displayName();
+
             if (displayName.isEmpty())
                 displayName = FileUtils::getFileDisplayName(info->uri());
             //when name is too long, show elideText
@@ -187,22 +228,50 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
             }
             if (info->isDir()) {
                 //add to bookmark option
-                l<<addAction(QIcon::fromTheme("bookmark-add-symbolic"), tr("Add to bookmark"));
-                connect(l.last(), &QAction::triggered, [=]() {
-                    //qDebug() <<"add to bookmark:" <<info->uri();
-                    auto bookmark = BookMarkManager::getInstance();
-                    if (bookmark->isLoaded()) {
-                        bookmark->addBookMark(info->uri());
-                    }
-                });
+                if (!info->isVirtual() &&  !info->uri().startsWith("smb://") && !m_is_kydroid)
+                {
+                    l<<addAction(QIcon::fromTheme("bookmark-add-symbolic"), tr("Add to bookmark"));
+                    connect(l.last(), &QAction::triggered, [=]() {
+                        //qDebug() <<"add to bookmark:" <<info->uri();
+                        auto bookmark = BookMarkManager::getInstance();
+                        if (bookmark->isLoaded()) {
+                            bookmark->addBookMark(info->uri());
+                        }
+                    });
+                }
 
-                l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open \"%1\"").arg(displayName));
+                l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("Open"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     if (!m_top_window)
                         return;
                     m_top_window->goToUri(m_selections.first(), true);
                 });
-                l<<addAction(QIcon::fromTheme("window-new-symbolic"), tr("Open \"%1\" in &New Window").arg(displayName));
+
+                auto recommendActions = FileLaunchManager::getRecommendActions(m_selections.first());
+                if (recommendActions.count() >1)
+                {
+                    auto openWithAction = addAction(tr("Open with..."));
+                    QMenu *openWithMenu = new QMenu(this);
+                    // do not highlight application icons.
+                    openWithMenu->setProperty("skipHighlightIconEffect", true);
+                    for (auto action : recommendActions) {
+                        action->setParent(openWithMenu);
+                        openWithMenu->addAction(static_cast<QAction*>(action));
+                    }
+                    auto fallbackActions = FileLaunchManager::getFallbackActions(m_selections.first());
+                    for (auto action : fallbackActions) {
+                        action->setParent(openWithMenu);
+                        openWithMenu->addAction(static_cast<QAction*>(action));
+                    }
+                    openWithMenu->addSeparator();
+                    openWithMenu->addAction(tr("More applications..."), [=]() {
+                        FileLauchDialog d(m_selections.first());
+                        d.exec();
+                    });
+                    openWithAction->setMenu(openWithMenu);
+                }
+
+                l<<addAction(QIcon::fromTheme("window-new-symbolic"), tr("Open in New Window"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     auto windowIface = m_top_window->create(m_selections.first());
                     auto newWindow = dynamic_cast<QWidget *>(windowIface);
@@ -210,21 +279,22 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
                     //FIXME: show when prepared?
                     newWindow->show();
                 });
-                l<<addAction(QIcon::fromTheme("tab-new-symbolic"), tr("Open \"%1\" in New &Tab").arg(displayName));
+                l<<addAction(QIcon::fromTheme("tab-new-symbolic"), tr("Open in New Tab"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     if (!m_top_window)
                         return;
                     m_top_window->addNewTabs(m_selections);
                 });
             } else if (!info->isVolume()) {
-                l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open \"%1\"").arg(displayName));
+                l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("Open"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     auto uri = m_selections.first();
                     FileLaunchManager::openAsync(uri, false, false);
                 });
-                auto openWithAction = addAction(tr("Open \"%1\" with...").arg(displayName));
-                //FIXME: add sub menu for open with action.
+                auto openWithAction = addAction(tr("Open with..."));
                 QMenu *openWithMenu = new QMenu(this);
+                // do not highlight application icons.
+                openWithMenu->setProperty("skipHighlightIconEffect", true);
                 auto recommendActions = FileLaunchManager::getRecommendActions(m_selections.first());
                 for (auto action : recommendActions) {
                     action->setParent(openWithMenu);
@@ -236,13 +306,13 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
                     openWithMenu->addAction(static_cast<QAction*>(action));
                 }
                 openWithMenu->addSeparator();
-                openWithMenu->addAction(tr("&More applications..."), [=]() {
+                openWithMenu->addAction(tr("More applications..."), [=]() {
                     FileLauchDialog d(m_selections.first());
                     d.exec();
                 });
                 openWithAction->setMenu(openWithMenu);
             } else {
-                l<<addAction(tr("&Open"));
+                l<<addAction(tr("Open"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     auto uri = m_selections.first();
                     //FIXME:
@@ -250,7 +320,7 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
                 });
             }
         } else {
-            l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open %1 selected files").arg(m_selections.count()));
+            l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("Open %1 selected files").arg(m_selections.count()));
             connect(l.last(), &QAction::triggered, [=]() {
                 qDebug()<<"triggered";
                 QStringList dirs;
@@ -263,21 +333,15 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
                     if (info->isDir() || info->isVolume()) {
                         dirs<<uri;
                     } else {
-                        QString mimeType = info->mimeType();
-                        if (mimeType.isEmpty()) {
-                            FileInfoJob job(info);
-                            job.querySync();
-                            mimeType = info->mimeType();
-                        }
-
+                        QString defaultAppName = FileLaunchManager::getDefaultAction(uri)->getAppInfoName();
                         QStringList list;
-                        if (fileMap.contains(mimeType)) {
-                            list = fileMap[mimeType];
+                        if (fileMap.contains(defaultAppName)) {
+                            list = fileMap[defaultAppName];
                             list << uri;
-                            fileMap.insert(mimeType, list);
+                            fileMap.insert(defaultAppName, list);
                         } else {
                             list << uri;
-                            fileMap.insert(mimeType, list);
+                            fileMap.insert(defaultAppName, list);
                         }
                     }
                 }
@@ -302,12 +366,25 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
 const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
 {
     QList<QAction *> l;
-    if (m_selections.isEmpty()) {
-        auto createAction = new QAction(tr("&New..."), this);
+    if (!m_is_favorite && m_selections.isEmpty()) {
+        auto createAction = new QAction(tr("New..."), this);
         if (m_is_cd) {
             createAction->setEnabled(false);
         }
-        if(m_directory.compare(QString::fromLocal8Bit("trash:///")) == 0)
+        //fix create folder fail issue in special path
+        auto info = FileInfo::fromUri(m_directory);
+        if (info.get()->isEmptyInfo()) {
+            //FIXME: replace BLOCKING api in ui thread.
+            FileInfoJob job(info);
+            job.querySync();
+        }
+
+        if (! info->canWrite())
+        {
+            createAction->setEnabled(false);
+        }
+
+        if(m_top_window->getFilterWorking())
         {
             createAction->setEnabled(false);
         }
@@ -317,59 +394,65 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
         addAction(createAction);
 
         //enumerate template dir
-        QDir templateDir(g_get_user_special_dir(G_USER_DIRECTORY_TEMPLATES));
-        auto templates = templateDir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot);
-        if (!templates.isEmpty()) {
-            for (auto t : templates) {
-                QFileInfo qinfo(templateDir, t);
-                GFile *gtk_file = g_file_new_for_path(qinfo.filePath().toUtf8().data());
-                char *uri_str = g_file_get_uri(gtk_file);
-                std::shared_ptr<FileInfo> info = FileInfo::fromUri(uri_str);
+//        QDir templateDir(g_get_user_special_dir(G_USER_DIRECTORY_TEMPLATES));
+        QString templatePath = GlobalSettings::getInstance()->getValue(TEMPLATES_DIR).toString();
+        if(!templatePath.isEmpty())
+        {
+            QDir templateDir(templatePath);
+            auto templates = templateDir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot);
+            if (!templates.isEmpty()) {
+                for (auto t : templates) {
+                    QFileInfo qinfo(templateDir, t);
+                    GFile *gtk_file = g_file_new_for_path(qinfo.filePath().toUtf8().data());
+                    char *uri_str = g_file_get_uri(gtk_file);
+                    std::shared_ptr<FileInfo> info = FileInfo::fromUri(uri_str);
 
-                QString mimeType = info->mimeType();
-                if (mimeType.isEmpty()) {
-                    FileInfoJob job(info);
-                    job.querySync();
-                    mimeType = info->mimeType();
-                }
-
-                QIcon tmpIcon;
-                GList *app_infos = g_app_info_get_recommended_for_type(mimeType.toUtf8().constData());
-                GList *l = app_infos;
-                QList<FileLaunchAction *> actions;
-                bool isOnlyUnref = false;
-                while (l) {
-                    auto app_info = static_cast<GAppInfo*>(l->data);
-                    if (!isOnlyUnref) {
-                        GThemedIcon *icon = G_THEMED_ICON(g_app_info_get_icon(app_info));
-                        const char * const * icon_names = g_themed_icon_get_names(icon);
-                        if (icon_names)
-                            tmpIcon = QIcon::fromTheme(*icon_names);
-                        if(!tmpIcon.isNull())
-                            isOnlyUnref = true;
+                    QString mimeType = info->mimeType();
+                    if (mimeType.isEmpty()) {
+                        FileInfoJob job(info);
+                        job.querySync();
+                        mimeType = info->mimeType();
                     }
-                    g_object_unref(app_infos);
-                    l = l->next;
-                }
 
-                QAction *action = new QAction(tmpIcon, qinfo.baseName(), this);
-                connect(action, &QAction::triggered, [=]() {
-                    CreateTemplateOperation op(m_directory, CreateTemplateOperation::Template, t);
-                    Peony::FileOperationErrorDialogConflict dlg;
-                    connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialogConflict::handle);
-                    op.run();
-                    auto target = op.target();
-                    m_uris_to_edit<<target;
-                });
-                subMenu->addAction(action);
-                g_free(uri_str);
-                g_object_unref(gtk_file);
+                    QIcon tmpIcon;
+                    GList *app_infos = g_app_info_get_recommended_for_type(mimeType.toUtf8().constData());
+                    GList *l = app_infos;
+                    QList<FileLaunchAction *> actions;
+                    bool isOnlyUnref = false;
+                    while (l) {
+                        auto app_info = static_cast<GAppInfo*>(l->data);
+                        if (!isOnlyUnref) {
+                            GThemedIcon *icon = G_THEMED_ICON(g_app_info_get_icon(app_info));
+                            const char * const * icon_names = g_themed_icon_get_names(icon);
+                            if (icon_names)
+                                tmpIcon = QIcon::fromTheme(*icon_names);
+                            if(!tmpIcon.isNull())
+                                isOnlyUnref = true;
+                        }
+                        l = l->next;
+                    }
+                    g_list_free_full(app_infos, g_object_unref);
+
+                    QAction *action = new QAction(tmpIcon, qinfo.baseName(), this);
+                    connect(action, &QAction::triggered, [=]() {
+                        // automatically check for conficts
+                        CreateTemplateOperation op(m_directory, CreateTemplateOperation::Template, t);
+                        Peony::FileOperationErrorDialogWarning dlg;
+                        connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialogWarning::handle);
+                        op.run();
+                        auto target = op.target();
+                        m_uris_to_edit<<target;
+                    });
+                    subMenu->addAction(action);
+                    g_free(uri_str);
+                    g_object_unref(gtk_file);
+                }
+                subMenu->addSeparator();
             }
-            subMenu->addSeparator();
         }
 
         QList<QAction *> actions;
-        auto createEmptyFileAction = new QAction(QIcon::fromTheme("document-new-symbolic"), tr("Empty &File"), this);
+        auto createEmptyFileAction = new QAction(QIcon::fromTheme("document-new-symbolic"), tr("Empty File"), this);
         actions<<createEmptyFileAction;
         connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory);
@@ -381,7 +464,7 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
             qDebug()<<"target:"<<targetUri;
             m_uris_to_edit<<targetUri;
         });
-        auto createFolderActions = new QAction(QIcon::fromTheme("folder-new-symbolic"), tr("&Folder"), this);
+        auto createFolderActions = new QAction(QIcon::fromTheme("folder-new-symbolic"), tr("Folder"), this);
         actions<<createFolderActions;
         connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory, nullptr, CreateTemplateOperation::EmptyFolder);
@@ -436,9 +519,9 @@ const QList<QAction *> DirectoryViewMenu::constructViewOpActions()
 
         QList<QAction *> tmp;
         tmp<<sortTypeMenu->addAction(tr("Name"));
-        tmp<<sortTypeMenu->addAction(tr("File Size"));
-        tmp<<sortTypeMenu->addAction(tr("File Type"));
         tmp<<sortTypeMenu->addAction(tr("Modified Date"));
+        tmp<<sortTypeMenu->addAction(tr("File Type"));
+        tmp<<sortTypeMenu->addAction(tr("File Size"));
         int sortType = m_view->getSortType();
         if (sortType >= 0) {
             tmp.at(sortType)->setCheckable(true);
@@ -515,38 +598,96 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
 
     if (!m_is_trash && !m_is_search && !m_is_computer) {
         QString homeUri = "file://" +  QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
-        if (!m_selections.isEmpty() && !m_selections.contains(homeUri)) {
-            l<<addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("&Copy"));
-            connect(l.last(), &QAction::triggered, [=]() {
-                ClipboardUtils::setClipboardFiles(m_selections, false);
-            });
-            l<<addAction(QIcon::fromTheme("edit-cut-symbolic"), tr("Cu&t"));
-            connect(l.last(), &QAction::triggered, [=]() {
-                ClipboardUtils::setClipboardFiles(m_selections, true);
-            });
-            l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("&Delete to trash"));
-            connect(l.last(), &QAction::triggered, [=]() {
-                FileOperationUtils::trash(m_selections, true);
-            });
-            //add delete forever option
-            l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
-            connect(l.last(), &QAction::triggered, [=]() {
-                FileOperationUtils::executeRemoveActionWithDialog(m_selections);
-            });
-            if (m_selections.count() == 1) {
-                l<<addAction(QIcon::fromTheme("document-edit-symbolic"), tr("&Rename"));
+        bool hasStandardPath = FileUtils::containsStandardPath(m_selections);
+        //qDebug() << "constructFileOpActions hasStandardPath:" <<hasStandardPath;
+        if (!m_selections.isEmpty() && !m_selections.contains(homeUri) && !m_is_recent) {
+            if (!m_is_favorite) {
+                //code from lixiang for kydroid menu
+                if (m_is_kydroid) {
+                    bool hasDir = false;
+                    for (auto uri : m_selections) {
+                        auto info = FileInfo::fromUri(uri);
+                        if (info->isDir()) {
+                            hasDir = true;
+                            break;
+                        }
+                    }
+                    if (!hasDir) {
+                        l<<addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("Copy"));
+                        connect(l.last(), &QAction::triggered, [=]() {
+                            ClipboardUtils::setClipboardFiles(m_selections, false);
+                        });
+                    }
+                }
+                else {
+                    l<<addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("Copy"));
+                    connect(l.last(), &QAction::triggered, [=]() {
+                        ClipboardUtils::setClipboardFiles(m_selections, false);
+                    });
+                }
+            }
+
+            //kydroid has no cut,delete,rename option
+            if (m_is_kydroid)
+                return l;
+
+            if (!hasStandardPath && !m_is_recent && !m_is_favorite)
+            {
+                l<<addAction(QIcon::fromTheme("edit-cut-symbolic"), tr("Cut"));
+                connect(l.last(), &QAction::triggered, [=]() {
+                    ClipboardUtils::setClipboardFiles(m_selections, true);
+                });
+            }
+
+            if (!m_is_recent && !m_is_favorite && !hasStandardPath) {
+                l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("Delete to trash"));
+                connect(l.last(), &QAction::triggered, [=]() {
+                    FileOperationUtils::trash(m_selections, true);
+                });
+            }
+
+            if (m_is_favorite && m_can_delete) {
+                l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
+                connect(l.last(), &QAction::triggered, [=]() {
+                    FileOperationUtils::executeRemoveActionWithDialog(m_selections);
+                });
+            }
+
+            if (m_selections.count() == 1 && ! hasStandardPath && !m_is_recent && !m_is_favorite) {
+                l<<addAction(QIcon::fromTheme("document-edit-symbolic"), tr("Rename"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     m_view->editUri(m_selections.first());
                 });
             }
         } else {
-            auto pasteAction = addAction(QIcon::fromTheme("edit-paste-symbolic"), tr("&Paste"));
-            l<<pasteAction;
-            pasteAction->setEnabled(ClipboardUtils::isClipboardHasFiles());
-            connect(l.last(), &QAction::triggered, [=]() {
-                ClipboardUtils::pasteClipboardFiles(m_directory);
-            });
-            l<<addAction(QIcon::fromTheme("view-refresh-symbolic"), tr("&Refresh"));
+            if (!m_is_recent && !m_is_favorite && !m_is_kydroid)
+            {
+                auto pasteAction = addAction(QIcon::fromTheme("edit-paste-symbolic"), tr("Paste"));
+                l<<pasteAction;
+                pasteAction->setEnabled(ClipboardUtils::isClipboardHasFiles());
+                connect(l.last(), &QAction::triggered, [=]() {
+                    auto op = ClipboardUtils::pasteClipboardFiles(m_directory);
+                    if (op) {
+                        auto window = dynamic_cast<QWidget *>(m_top_window);
+                        auto iface = m_top_window;
+                        connect(op, &Peony::FileOperation::operationFinished, window, [=](){
+                            auto opInfo = op->getOperationInfo();
+                            auto targetUirs = opInfo->dests();
+                            iface->setCurrentSelectionUris(targetUirs);
+                        }, Qt::BlockingQueuedConnection);
+                    }
+                });
+            }
+            else if (m_is_recent)
+            {
+                //fix recent files can not be deleted issue
+                l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
+                connect(l.last(), &QAction::triggered, [=]() {
+                    FileOperationUtils::executeRemoveActionWithDialog(m_selections);
+                });
+            }
+
+            l<<addAction(QIcon::fromTheme("view-refresh-symbolic"), tr("Refresh"));
             connect(l.last(), &QAction::triggered, [=]() {
                 m_top_window->refresh();
             });
@@ -556,13 +697,13 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
     //select all and reverse select
     if (m_selections.isEmpty())
     {
-        l<<addAction(tr("Select &All"));
+        l<<addAction(tr("Select All"));
         connect(l.last(), &QAction::triggered, [=]() {
             //qDebug() << "select all";
             m_view->invertSelections();
         });
     }
-    else
+    else if(! m_is_kydroid)
     {
         l<<addAction(tr("Reverse Select"));
         connect(l.last(), &QAction::triggered, [=]() {
@@ -578,8 +719,12 @@ const QList<QAction *> DirectoryViewMenu::constructFilePropertiesActions()
 {
     QList<QAction *> l;
 
-    if (!m_is_search) {
-        l<<addAction(QIcon::fromTheme("preview-file"), tr("P&roperties"));
+    //fix select mutiple file in trash path show empty issue
+    if (m_selections.count() >1 && m_is_trash)
+        return l;
+
+    if (! m_is_search) {
+        l<<addAction(QIcon::fromTheme("preview-file"), tr("Properties"));
         connect(l.last(), &QAction::triggered, [=]() {
             //FIXME:
             if (m_selections.isEmpty()) {
@@ -595,7 +740,7 @@ const QList<QAction *> DirectoryViewMenu::constructFilePropertiesActions()
             }
         });
     } else if (m_selections.count() == 1) {
-        l<<addAction(QIcon::fromTheme("preview-file"), tr("P&roperties"));
+        l<<addAction(QIcon::fromTheme("preview-file"), tr("Properties"));
         connect(l.last(), &QAction::triggered, [=]() {
             PropertiesWindow *p = new PropertiesWindow(m_selections);
             p->setAttribute(Qt::WA_DeleteOnClose);
@@ -642,6 +787,7 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
             l<<addAction(QIcon::fromTheme("window-close-symbolic"), tr("&Clean the Trash"));
             l.last()->setEnabled(!isTrashEmpty);
             connect(l.last(), &QAction::triggered, [=]() {
+                AudioPlayManager::getInstance()->playWarningAudio();
                 auto result = QMessageBox::question(nullptr, tr("Delete Permanently"), tr("Are you sure that you want to delete these files? "
                                                                                           "Once you start a deletion, the files deleting will never be "
                                                                                           "restored again."));
@@ -651,7 +797,7 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
                 }
             });
         } else {
-            l<<addAction(tr("&Restore"));
+            l<<addAction(tr("Restore"));
             connect(l.last(), &QAction::triggered, [=]() {
                 if (m_selections.count() == 1) {
                     FileOperationUtils::restore(m_selections.first());
@@ -659,8 +805,9 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
                     FileOperationUtils::restore(m_selections);
                 }
             });
-            l<<addAction(QIcon::fromTheme("window-close-symbolic"), tr("&Delete"));
+            l<<addAction(QIcon::fromTheme("window-close-symbolic"), tr("Delete"));
             connect(l.last(), &QAction::triggered, [=]() {
+                AudioPlayManager::getInstance()->playWarningAudio();
                 auto result = QMessageBox::question(nullptr, tr("Delete Permanently"), tr("Are you sure that you want to delete these files? "
                                                                                           "Once you start a deletion, the files deleting will never be "
                                                                                           "restored again."));
@@ -669,6 +816,11 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
                 }
             });
         }
+    } else if (m_is_recent && m_selections.isEmpty()) {
+        l<<addAction(tr("Clean All"));
+        connect(l.last(), &QAction::triggered, [=]() {
+            RecentVFSManager::getInstance()->clearAll();
+        });
     }
 
     return l;
@@ -677,15 +829,19 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
 const QList<QAction *> DirectoryViewMenu::constructSearchActions()
 {
     QList<QAction *> l;
-    if (m_is_search) {
+    if (m_is_search || m_is_recent) {
         if (m_selections.isEmpty())
             return l;
 
         l<<addAction(QIcon::fromTheme("new-window-symbolc"), tr("Open Parent Folder in New Window"));
         connect(l.last(), &QAction::triggered, [=]() {
             for (auto uri : m_selections) {
+                if (m_is_recent)
+                    uri = FileUtils::getTargetUri(uri);
                 auto parentUri = FileUtils::getParentUri(uri);
-                if (!parentUri.isNull()) {
+                //bool exist = FileUtils::isFileExsit(uri);
+                bool exist = true;
+                if (exist && ! parentUri.isNull()) {
                     auto *windowIface = m_top_window->create(parentUri);
                     auto newWindow = dynamic_cast<QWidget *>(windowIface);
                     auto selection = m_selections;
@@ -701,6 +857,12 @@ const QList<QAction *> DirectoryViewMenu::constructSearchActions()
                     });
 #endif
                     newWindow->show();
+                 }
+                else
+                {
+                    QMessageBox::warning(nullptr,
+                                         tr("Error"),
+                                         tr("File:\"%1\ is not exist, did you moved or deleted it?").arg(QUrl(uri).path()));
                 }
             }
         });
@@ -711,17 +873,19 @@ const QList<QAction *> DirectoryViewMenu::constructSearchActions()
 const QList<QAction *> DirectoryViewMenu::constructMenuPluginActions()
 {
     QList<QAction *> l;
-    auto pluginIds = MenuPluginManager::getInstance()->getPluginIds();
-    //sort plugiins by name, so the menu option orders is relatively fixed
-    qSort(pluginIds.begin(), pluginIds.end());
+    if (!m_is_favorite) {
+        auto pluginIds = MenuPluginManager::getInstance()->getPluginIds();
+        //sort plugiins by name, so the menu option orders is relatively fixed
+        std::sort(pluginIds.begin(), pluginIds.end());
 
-    for (auto id : pluginIds) {
-        auto plugin = MenuPluginManager::getInstance()->getPlugin(id);
-        auto actions = plugin->menuActions(MenuPluginInterface::DirectoryView, m_directory, m_selections);
-        l<<actions;
-        for (auto action : actions) {
-            action->setParent(this);
-            addAction(action);
+        for (auto id : pluginIds) {
+            auto plugin = MenuPluginManager::getInstance()->getPlugin(id);
+            auto actions = plugin->menuActions(MenuPluginInterface::DirectoryView, m_directory, m_selections);
+            l<<actions;
+            for (auto action : actions) {
+                action->setParent(this);
+                addAction(action);
+            }
         }
     }
     return l;
