@@ -26,6 +26,8 @@
 
 #include <QAction>
 #include <QCompleter>
+#include <QVector4D>
+
 #include <QDebug>
 
 using namespace Peony;
@@ -37,20 +39,20 @@ SearchBarContainer::SearchBarContainer(QWidget *parent): QWidget(parent)
     m_layout = layout;
     layout->setContentsMargins(0,0,0,0);
 
-    QComboBox *filter = new QComboBox(this);
-    m_filter_box = filter;
-    filter->setToolTip(tr("Choose File Type"));
+//    QComboBox *filter = new QComboBox(this);
+//    m_filter_box = filter;
+//    filter->setToolTip(tr("Choose File Type"));
     auto model = new QStringListModel(this);
     model->setStringList(m_file_type_list);
-    filter->setModel(model);
-    filter->setFixedWidth(80);
-    filter->setFixedHeight(parent->height());
+//    filter->setModel(model);
+//    filter->setFixedWidth(80);
+//    filter->setFixedHeight(parent->height());
 
     QLineEdit *edit = new QLineEdit(this);
     m_search_box = edit;
     edit->setFixedHeight(parent->height());
 
-    layout->addWidget(filter, Qt::AlignLeft);
+//    layout->addWidget(filter, Qt::AlignLeft);
     layout->addWidget(edit, Qt::AlignLeft);
 
     //search history
@@ -63,8 +65,19 @@ SearchBarContainer::SearchBarContainer(QWidget *parent): QWidget(parent)
     m_list.prepend(tr("Clear"));
     m_model->setStringList(m_list);
     m_list_view = new QListView(m_search_box);
+
+    m_list_view->setAttribute(Qt::WA_TranslucentBackground);
+    //m_list_view->viewport()->setAttribute(Qt::WA_TranslucentBackground);
+
+    m_list_view->setProperty("useCustomShadow", true);
+    m_list_view->setProperty("customShadowDarkness", 0.5);
+    m_list_view->setProperty("customShadowWidth", 20);
+    m_list_view->setProperty("customShadowRadius", QVector4D(6, 6, 6, 6));
+    m_list_view->setProperty("customShadowMargins", QVector4D(20, 20, 20, 20));
+
     m_list_view->setModel(m_model);
     completer->setPopup(m_list_view);
+
     //change QCompleter Mode form UnfilteredPopupCompletion to PopupCompletion
     //to fix can not input chinese continuous issue
     completer->setCompletionMode(QCompleter::PopupCompletion);
@@ -78,12 +91,15 @@ SearchBarContainer::SearchBarContainer(QWidget *parent): QWidget(parent)
         {
             m_search_trigger.stop();
         }
-        m_search_trigger.start();
-    });
-    connect(m_filter_box, &QComboBox::currentTextChanged, [=]()
-    {
-        Q_EMIT this->filterUpdate(m_filter_box->currentIndex());
+        if (! m_clear_action)
+            m_search_trigger.start();
+        else
+            m_clear_action = false;
     });
+//    connect(m_filter_box, &QComboBox::currentTextChanged, [=]()
+//    {
+//        Q_EMIT this->filterUpdate(m_filter_box->currentIndex());
+//    });
 
     QAction *searchAction = m_search_box->addAction(QIcon::fromTheme("go-down"), QLineEdit::TrailingPosition);
     connect(searchAction, &QAction::triggered, this, [=]() {
@@ -128,4 +144,5 @@ void SearchBarContainer::startSearch()
 void SearchBarContainer::clearSearchBox()
 {
     m_search_box->setText("");
+    m_clear_action = true;
 }
